/* -*- mode:prolog; -*-
 ******************************************************************
 * $Id$
 * Copyright (C) 2008 by INRIA 
 * Author: Eric de la Clergerie <Eric.De_La_Clergerie@inria.fr>
 * ----------------------------------------------------------------
 *
 *  fst.pl -- Implementation jouet des FST
 *
 * ----------------------------------------------------------------
 * Description
 *
 *   Implementation jouet des transducteurs  tats finis (FST) en
 *   Prolog/DyALogImplementation jouet des automates  tats finis
 *   (FSA) en Prolog/DyALog
 *
 * ----------------------------------------------------------------
 */

%% Import de la bibliothque DyALog pour l'impression
:-require 'format.pl'.

%% Directive extensional pour dfinir des prdicats bases de donnes
%% Ici, les prdicats extensionels dfinissant tats et transitions
%% d'un FSA
:-extensional trans/4.
:-extensional init_state/2.
:-extensional final_state/2.

%% Coeur du moteur FST
fst(Name,L1,R1,L2,R2) :-
	init_state(Name,Init_State),
	fst(Name,Init_State,L1,R1,L2,R2).

fst(Name,S,L1,L1,L2,L2) :- final_state(Name,S).
fst(Name,S,L1,R1,L2,R2) :-
	trans(Name,S,T,(A1:A2)),
	fst_action(A1,L1,M1),
	fst_action(A2,L2,M2),
	fst(Name,T,M1,R2,M2,R2)
	.

%% Une nouvelle action 
fst(Name,S,L1,R1,L2,R2) :-
	trans(Name,S,T,A),
	fst_sync_action(A,L1,M1,L2,M2),
	fst(Name,T,M1,R1,M2,R2)
	.

%% Les action du FST
fst_action(c(X),L,R) :- 'C'(L,X,R).
fst_action(true,L,L).
fst_action(range(D),L,R) :- 'C'(L,X,R),domain(X,D).


fst_sync_action(srange(D),L1,R1,L2,R2) :-
	'C'(L1,X1,R1),
	'C'(L2,X2,R2),
	domain(X1:X2,D).


%% Le lecteur sur liste (en complment du lecteur sur treillis de mots)
:-extensional 'C'/3.
'C'([C|R],C,R).

%% Une requte possible
%% Lecture sur une liste prolog de symboles
?-argv([Name|L1]),fst(Name,L1,[],L2,[]).

%% Inverser les bandes
?-argv(['-rev',Name|SL2]),
  atoms2nums(SL2,L2),
  fst(Name,L1,[],L2,[])
  .

%% Tout generer
?-argv(['-gen',Name]),
  fst(Name,L1,[],L2,[])
  .

:-rec_prolog atoms2nums/2.

atoms2nums([],[]).
atoms2nums([Smb|SL],[Num|L]) :-
	atom_number(Smb,Num),
	atoms2nums(SL,L).


