/* -*- mode:prolog; -*-
 ******************************************************************
 * $Id$
 * Copyright (C) 2008 by INRIA 
 * Author: Eric de la Clergerie <Eric.De_La_Clergerie@inria.fr>
 * ----------------------------------------------------------------
 *
 *  regexp2fst.pl -- Compilateur Regexp 2 bandes vers FST
 *
 * ----------------------------------------------------------------
 * Description
 *
 * Petit exemple de compilateur d'expressions rgulire 2 bandes
 *  en transducteurs  tats finis
 *
 * ----------------------------------------------------------------
 */

%% Import de la bibliothque DyALog pour l'impression
:-require 'format.pl'.

%% Dclaration des oprateurs pour construire les regexp
:-op(yf,['@?'],500).
:-op(yf,['@+'],500).
:-op(yf,['@*'],500).

%% ** Spcifique DyALog **
%% Pour ne pas tabuler le prdicat regexp2fst
%% et avoir un comportement standard a la Prolog
:-rec_prolog regexp2fst/4.

%% Dconstruire les constructeurs de regexp

%% Concatnation
regexp2fst(Name,(E1,E2),S1,S2) :-
	regexp2fst(Name,E1,S1,S3),
	regexp2fst(Name,E2,S3,S2)
	.

%% Alternative
regexp2fst(Name,(E1;E2),S1,S2) :-
	regexp2fst(Name,E1,S1,S2),
	regexp2fst(Name,E2,S1,S2)
	.

%% Optionnel
regexp2fst(Name,E @?,S1,S2) :-
	regexp2fst(Name,E,S1,S2),
	emit_trans(Name,S1,S2,true:true)
	.

%% toile de Kleene
regexp2fst(Name,E @*,S1,S2) :-
	regexp2fst(Name,E,S1,S1),
	regexp2fst(Name,true:true,S1,S2)
	.

%% toile de Kleene stricte
regexp2fst(Name,E @+,S1,S2) :-
	regexp2fst(Name,E,S1,S1),
	regexp2fst(Name,true:true,S1,S2)
	.

%% Expressions de base
regexp2fst(Name,A1:A2,S1,S2) :-
	domain(A1,[c(_),range(_),true]),
	domain(A2,[c(_),range(_),true]),
	( var(S2) ->
	  mutable_new_state(Name,S2)
	;
	  true
	),
	emit_trans(Name,S1,S2,A1:A2).

regexp2fst(Name,srange(D),S1,S2) :-
	( var(S2) ->
	  mutable_new_state(Name,S2)
	;
	  true
	),
	emit_trans(Name,S1,S2,srange(D)).

%% Prdicat prolog-like pour l'mission des transitions
:-rec_prolog emit_trans/4.

emit_trans(Name,S1,S2,A) :-
	format('trans(~w,~w,~w,~q).\n',[Name,S1,S2,A])
	.


%% Generation des tats avec des prdicats non logiques
:-std_prolog mutable_init_state/2.

mutable_init_state(Name,0) :-
	mutable(MX,1),
	record(state(Name,MX))
	.

:-std_prolog mutable_new_state/2.

mutable_new_state(Name,S) :-
	recorded(state(Name,MX)),
	mutable_inc(MX,S),
	true
	.

%% Le prdicat base de donnes pour accder aux regexp
:-extensional rx2/2.

%% La requte chapeau pour la compilation
?-rx2(Name,RX),
  %% mission d'un petit header
  format('%% Compilation de la regexp 2 bandes ~w\n\n',[Name]),
  %% traitement de l'expression rgulire
  mutable_init_state(Name,Init),
  regexp2fst(Name,RX,Init,Final),
  %% mission des tats initiaux et finaux
  format('init_state(~w,~w).\n',[Name,Init]),
  format('final_state(~w,~w).\n',[Name,Final]),
  fail.
